<?php

namespace Drupal\omni_twig\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Field\FieldItemListInterface;

/**
 * Plugin implementation of the 'omni_twig' formatter.
 *
 * @FieldFormatter(
 *   id = "omni_twig",
 *   label = @Translation("Twig Template"),
 *   field_types = {}
 * )
 */
class OmniTwig extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    return [$this->t('Render the field using a Twig Template.')];
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'template' => implode("\n", [
        '{# Uses the default formatter, but that can be overridden. #}',
        '<div class="{{ field_class }}">',
        '  {% if not label_hidden %}',
        '    <div class="{{ title_class }}">{{ title }}</div>',
        '  {% endif %}',
        '  {% for item in items %}',
        '    <div class="{{ item_class }}">{{ item|view }}</div>',
        '  {% endfor %}',
        '</div>',
      ]),
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    return [
      'template' => [
        '#title' => $this->t('Twig Template'),
        '#type' => 'textarea',
        '#default_value' => $this->getSetting('template'),
        '#description' => 'Additional Features:<ul><li>Variables: items [all field items], field [the first field item], field_class, title, title_class, label_hidden, label_display, item_class, entity [of this field], parent [entity], bundle, name, field_definition.</li><li>Functions: dump(variable).</li><li>Filters: view(name, settings).</li></ul>',
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function view(FieldItemListInterface $items, $langcode = NULL) {
    // Default the language to the current content language.
    if (empty($langcode)) {
      $langcode = \Drupal::languageManager()->getCurrentLanguage(LanguageInterface::TYPE_CONTENT)->getId();
    }

    $definition = $items->getFieldDefinition();
    $entity = $items->getEntity();
    $elements = [
      '#type' => 'inline_template',
      '#template' => $this->getSetting('template'),
      '#context' => [
        'langcode' => $langcode,
        'label_display' => $this->label,
        'label_hidden' => $this->label === 'hidden',
        'title_class' => [
          'above' => 'field--label',
          'inline' => 'field--label',
          'hidden' => 'field--label d-none',
          'visually_hidden' => 'field--label sr-only',
        ][$this->label],
        'field_class' => implode(' ', [
          'field',
          'field--name-' . $this->fieldDefinition->getName(),
          'field--type-' . $this->fieldDefinition->getType(),
          'field--label-' . $this->label,
        ]),
        'item_class' => 'field--item',
        'label' => $definition->getLabel(),
        'title' => $definition->getLabel(),
        'parent' => $items->getParent()->getEntity(),
        'entity' => $entity,
        'bundle' => $entity->bundle(),
        'name' => $items->getName(),
        'field_definition' => $definition,
        'items' => $items,
        'field' => $items->get(0),
      ],
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {}

}
